% Computing optimal decision rules in a consumption saving problem with
% income given by two processes, one AR and one temporary (which mimics the
% SHIW question)
% needs markovappr.m mprocesses.m japista_hwd.m
% produces excel file mpcmod.xls which is then used by stata files 

clear all;

% Set parameters

npa=2000;  % number of endo states
amin=-1; % minimum wealth  
amax=15; % maximum wealth

bita=0.95; % discount factor
R=1.04;   %  gross interest rate
sigma=2.0;  % relative risk aversion


pers1=0.98  % persistence of AR1
sd1=0.01;   % sd of innovations
N1=9;       % number of states of mark chain


pers2=0;    % persistence of second process
sd2=0.03;   % sd of innovations
N2=3;       % number of states of mark chain


Avec=linspace(amin,amax,npa); % make a grid on assets

mprocesses  % call the routine that create Markov chains
load matrices



v=ones(nvz,npa);
c=ones(1,npa);

V = ones(nvz,npa)/(1-bita);



diffV   = 1;
iter    = 1;

% Initialize choices and value
newV = V;
EV = tmatvz*V;
idecS = zeros(nvz,npa);
Avecp = Avec ;


while (iter <= 5000) & (diffV > 1e-6)

    % Compute expected future value conditional on exo state state today, z
    % and choice of a' i.e. E(V(a',z')=f(a',z) 
    
    
    EV=tmatvz*V;


     % compute values for every possible value of aprime
    for iz = 1:nvz
        for ia = 1:npa

            c = max( 1e-200, vy(iz) - Avecp + R*Avec(ia) ) ;
            v = (1/(1-sigma))*c.^(1-sigma) + bita*EV(iz,:) ;
 
            % "c", "v", EV(iz,:) are vectors of size [1,na], functions of a' 
            % We look in each state for the a' that maximizes v
            
            % new value function and index of decision rules
            [newV(iz,ia), idecA(iz,ia)] = max ( v ) ;
          
        end
    end

    japista_hwd  % howard policy iteration
    
    diffV = max(abs((newV(:)-V(:))./newV(:)));
    iter  = iter + 1;
    V     = newV ;
    disp(diffV);

end

decA = Avec(idecA);  % decision rules for assets



disp('Decision rules computed')
[ Ym Am ] = ndgrid(vy,Avec);
% consumption is a grid too of dimension nz,ns
decC = Ym - decA + R*Am ;



% now compute accumulation and consumption function for all possible values
% of cash in hand and temp shock =0 (j=2)
j=2;
ii=1;
for i=1:N1
    for na=1:npa
        cih0(ii)=Avec(na)*R+vy((j-1)*N1+i);
        aprime0(ii)=decA((j-1)*N1+i,na);
        cons0(ii)=decC((j-1)*N1+i,na);
        ii=ii+1;
    end
end



j=3;
ii=1;
for i=1:N1
    for na=1:npa
        cih1(ii)=Avec(na)*R+vy((j-1)*N1+i);
        aprime1(ii)=decA((j-1)*N1+i,na);
        cons1(ii)=decC((j-1)*N1+i,na);
        ii=ii+1;
    end
end

check=(cih1-cih0)-(cons1-cons0)-(aprime1-aprime0);

plot(cih0,(cons1-cons0)./(cih1-cih0),' o')
title('MPC as a function of cash in hand');



delete mpcmod.xls
xlswrite('mpcmod.xls',[ cih0' (cons1-cons0)'./(cih1-cih0)' ])
